 /*                                   */
 /*      Zak's H8 Monitor C code      */
 /*      memory.c                     */
 /*                                   */
 /*   2001-2002 all rights reserved   */
 /*          by Kazutaka 'Zak' Sawa   */
 /*                                   */


#include "zTypes.h"
#include "utilities.h"
#include "memory.h"
#include "error.h"


static const SInt8*		kMemoryDumpTitle
															 = "Address :00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F:0123456789ABCDEF";
static const SInt8*		kMemoryDumpLine
															 = "========:===============================================:================";


/* [_v\ */
extern void	doDumpMemory(void)
	{
		/*var*/
		UInt32							dumpStartAddr;
		UInt32							dumpLength;
		UInt32							startAddr;
		UInt32							lastAddress;
		SInt8								tempString[16];
		SInt16							byteCounter;
		Boolean							firstTitle;
		SInt8								oneChar;
		
		/*functon*/
		/* p[^`FbN */
		if (gDMem.NumWords != 3)
			{
				showError(kNumParamError);
				return;
			}
			
		/* p[^`FbN */
		if (! Hex2Num(&(gDMem.CurrentWords[1][0]), &dumpStartAddr))
			{
				showError(kHexNumError);
				return;
			}
		
		if (! Hex2Num(&(gDMem.CurrentWords[2][0]), &dumpLength))
			{
				showError(kHexNumError);
				return;
			}
		
		/* [_v\ */
		startAddr = dumpStartAddr & 0xFFFFFFF0;
		lastAddress = dumpStartAddr + dumpLength;
		firstTitle = true;
		
		for (; ; )
			{
				if (firstTitle || (startAddr == (startAddr & 0xFFFFFF00)))
					{
						if (! firstTitle)
							{
								sendString(kMemoryDumpLine, true);
								sendString(NULL, true);
							}
						sendString(kMemoryDumpTitle, true);
						sendString(kMemoryDumpLine, true);
						firstTitle = false;
					}
				
				Num2Hex(startAddr, 8, tempString);
				tempString[8] = ':';
				tempString[9] = 0x00;
				sendString(tempString, false);
				
				for (byteCounter = 0; byteCounter < 16; byteCounter++)
					{
						if ((startAddr >= dumpStartAddr) && (startAddr < lastAddress))
							Num2Hex(*((UInt8*)startAddr), 2, tempString);
						else
							{
								tempString[0] = ' ';
								tempString[1] = ' ';
							}
							
						if (byteCounter < 15)
							{
								tempString[2] = ' ';
								tempString[3] = 0x00;
							}
						else
							tempString[2] = 0x00;
							
						sendString(tempString, false);
						startAddr++;
					}
					
				sendChar(':');
				startAddr -= 16;
				
				for (byteCounter = 0; byteCounter < 16; byteCounter++)
					{
						if ((startAddr >= dumpStartAddr) && (startAddr < lastAddress))
							{
								oneChar = *((UInt8*)startAddr);
								if ((oneChar > ' ') && (oneChar < '~'))
									sendChar(oneChar);
								else
									sendChar('.');
							}
						else
							sendChar(' ');
						startAddr++;
					}
					
				sendString(NULL, true);
				
				if (startAddr >= lastAddress)
					break;
			}
		sendString(kMemoryDumpLine, true);
	}

	
/* [\ RA */
static void	doShowMemory
		 (const SInt16			showLength)						/* \oCg */
	{
		/*var*/
		UInt32							showAddr;
		SInt8								tempString[16];
		UInt32							theData;
		
		/*functon*/
		/* p[^`FbN */
		if (gDMem.NumWords != 2)
			{
				showError(kNumParamError);
				return;
			}
			
		/* p[^`FbN */
		if (! Hex2Num(&(gDMem.CurrentWords[1][0]), &showAddr))
			{
				showError(kHexNumError);
				return;
			}
		
		if ((showLength != 1) && (showAddr & 0x00000001))
			{
				showError(kOddAddressError);
				return;
			}
		
		/* [\ */
		Num2Hex(showAddr, 8, tempString);
		tempString[8] = ' ';
		tempString[9] = '=';
		tempString[10] = ' ';
		tempString[11] = 0x00;
		sendString(tempString, false);
		
		switch (showLength)
			{
				case 4:
					theData = *((UInt32*)showAddr);
					break;
					
				case 2:
					theData = *((UInt16*)showAddr);
					break;
					
				case 1:
				default:
					theData = *((UInt8*)showAddr);
					break;
			}
		
		Num2Hex(theData, showLength * 2, tempString);
		sendString(tempString, true);
	}

	
/* [\ Gg */
extern void	doShowMemory32(void)
	{
		doShowMemory(4);
	}

extern void	doShowMemory16(void)
	{
		doShowMemory(2);
	}

extern void	doShowMemory8(void)
	{
		doShowMemory(1);
	}

	
/* [ RA */
static void	doSetMemory
		 (const SInt16			setLength)						/* ނoCg */
	{
		/*var*/
		UInt32							setAddr;
		UInt32							setData;
		
		/*functon*/
		/* p[^`FbN */
		if (gDMem.NumWords != 3)
			{
				showError(kNumParamError);
				return;
			}
			
		/* p[^`FbN */
		if (! Hex2Num(&(gDMem.CurrentWords[1][0]), &setAddr))
			{
				showError(kHexNumError);
				return;
			}
		
		if (! Hex2Num(&(gDMem.CurrentWords[2][0]), &setData))
			{
				showError(kHexNumError);
				return;
			}
		
		if ((setLength != 1) && (setAddr & 0x00000001))
			{
				showError(kOddAddressError);
				return;
			}
		
		/* [ */
		switch (setLength)
			{
				case 4:
					*((UInt32*)setAddr) = setData;
					break;
					
				case 2:
					if (setData < 0x00010000L)
						*((UInt16*)setAddr) = setData;
					else
						showError(kTooLargeDataError);
					break;
					
				case 1:
				default:
					if (setData < 0x00000100L)
						*((UInt8*)setAddr) = setData;
					else
						showError(kTooLargeDataError);
					break;
			}
	}

	
/* [ Gg */
extern void	doSetMemory32(void)
	{
		doSetMemory(4);
	}

extern void	doSetMemory16(void)
	{
		doSetMemory(2);
	}

extern void	doSetMemory8(void)
	{
		doSetMemory(1);
	}

