 /*                                   */
 /*      Zak's H8 Monitor C code      */
 /*      monitor.c                    */
 /*                                   */
 /*   2001-2009 all rights reserved   */
 /*          by Kazutaka 'Zak' Sawa   */
 /*                                   */


#include "36064s.h"				/* I/OANZXpCN[ht@C */

#include "zTypes.h"
#include "monitor.h"
#include "initialize.h"
#include "utilities.h"
#include "help.h"
#include "memory.h"


typedef void(*comProcPtr)(void);

typedef struct
	{
		SInt8				command[kWordSize];
		comProcPtr			JumpAddress;
	}
	commandArray;

#define	kNumCommands		12
static const commandArray	CommandTable[kNumCommands]
										= {{"?",			doHelp},
											 {"HELP",		doHelp},
											 {"DM",			doDumpMemory},
											 {"DL",			doShowMemory32},
											 {"DW",			doShowMemory16},
											 {"DB",			doShowMemory8},
											 {"SL",			doSetMemory32},
											 {"SW",			doSetMemory16},
											 {"SB",			doSetMemory8}};

#define kTabCode			0x09

#define	kWelcomeLines		8
static const SInt8		kWelcomeMessage[kWelcomeLines][42]
										 = {"+=======================================+",
											"| << Welcome to the Zak's H8 monitor >> |",
											"|                (for 36064) ver. 0.2.0 |",
											"|                            2009/02/27 |",
											"|                                       |",
											"|  all rights reserved                  |",
											"|                by Kazutaka 'Zak' Sawa |",
											"+=======================================+"};
//static const SInt8*		kWelcomeLCD1	= "Zak's H8 monitor";
//static const SInt8*		kWelcomeLCD2	= "      ver. 0.2.0";

static const SInt8*		kPrompt = ">";
static const SInt8		kBackSP[4] = {0x08, 0x20, 0x08, 0x00};

static const SInt8*		kErrorNotSupportCommand	= " is not supported.";

DataMemRec				gDMem;				// master global data


/* EGJbZ[WM */
static void	sendWelcome(void)
	{
		/*var*/
		SInt16							counter;
	
		/*functon*/
		/* for Terminal */
		sendString(NULL, true);
		for (counter = 0; counter < kWelcomeLines; counter++)
			sendString(&(kWelcomeMessage[counter][0]), true);
		sendString(NULL, true);

		/* for LCD */
//		showLCDString(kWelcomeLCD1, 0);
//		showLCDString(kWelcomeLCD2, 1);
	}


/* ȈՃqXg */
static void	doHistry(void)
	{
		/*var*/
		SInt16							counter;
		SInt16							LastCommandLen;
	
		/*functon*/
		/* ݓ͂Ă镶̏ */
		if (gDMem.CommandLength > 0)
			for (counter = gDMem.CommandLength; counter > 0; counter--)
				sendString(kBackSP, false);	
		
		/* O͕̓obt@ɃRs[ */
		LastCommandLen = StringLength(gDMem.LastCommandLine);
		BlockMove(gDMem.CurrentCommandLine, gDMem.LastCommandLine, LastCommandLen);
		gDMem.CommandLength = LastCommandLen;
		
		/* ʂɔf */
		sendString(gDMem.LastCommandLine, false);
	}


/* L[͎CSIR[hu` */
static void	doCSICode
		 (SInt8							oneChar)							/* ͂P */
	{
		/*functon*/
		switch(oneChar)
			{
				case 'A':		/* J[\ Up */
					doHistry();
					break;
		
				case 'B':		/* J[\ Down */
					break;
		
				case 'C':		/* J[\ Right */
					break;
		
				case 'D':		/* J[\ Left */
					break;
		
				default:
					break;
			}
	}


/* L[͎̃Rg[R[h */
static void	doControlCode
		 (SInt8							oneChar,				/* ͂P */
			Boolean*					getCommand,				/* R}h̎swꂽtrue */
			Boolean*					nowEsc,					/* ݃GXP[vV[PX̎true */
			Boolean*					nowCSI)					/* CSI̎true */
	{
		/*functon*/
		*getCommand = false;
		
		switch(oneChar)
			{
				case 0x0D:
				case 0x0A:
					if (gDMem.CommandLength > 0)	/* 0D0A0A0Ď㔼̂Ă邽߂ɊmF */
						{
							gDMem.CurrentCommandLine[gDMem.CommandLength] = 0x00;
							BlockMove(gDMem.CurrentCommandLine, gDMem.LastCommandLine, kCommandBufferSize);
							*getCommand = true;
						}
					break;
			
				case 0x1B:
					*nowEsc = true;
					break;
			
				case 0x08:
				case 0x7F:
					if (gDMem.CommandLength > 0)
						{
			/*				oneChar = 0x7F;				  DELɓ 
							sendChar(oneChar);										*/
							gDMem.CommandLength--;
							sendString(kBackSP, false); 
						}
					break;
				
				default:
					if (*nowEsc)
						{				/* GXP[vV[PX */
							if (*nowCSI)
								{
									doCSICode(oneChar);
									*nowEsc = false;
									*nowCSI = false;
								}
							else
								switch(oneChar)
									{
										case '[':
											*nowCSI = true;
											break;
								
										default:
											*nowEsc = false;
											*nowCSI = false;
											break;
									}
						}
					break;
			}
	}


/* L[͑҂[v */
static void	commandWait
		 (UInt16*					ButtonCommand)				/* ꂽXCb` */
{
	/*functon*/
	/* R}h͑҂ */
	gDMem.CommandLength = 0;
	*ButtonCommand = 0x00;
	
	for ( ; ;)
		{
			/* {^ */
			if (kBoardButton == 0)
				{
					*ButtonCommand = 0x01;
					return;
				}
				
			/* ^[~i͏ */
			if (gDMem.CurrentRecievePos != gDMem.CurrentCopyedPos)
				{	
					/*var*/
					SInt8					oneChar;
					Boolean					getCommand = false;
				 	Boolean					nowEsc = false;
					Boolean					nowCSI = false;

					/*functon*/
					SCI3.SCR3.BIT.RIE = 0;		/* M֎~ԊJn */
					
					for (; ((gDMem.CurrentRecievePos != gDMem.CurrentCopyedPos) && (! getCommand)); )
						{
							/* Mobt@PoCgm & Rs[|WVXV*/
							oneChar = gDMem.RecieveBuffer[gDMem.CurrentCopyedPos++];
							if (gDMem.CurrentCopyedPos >= kRecieveRingBufferSize)
								gDMem.CurrentCopyedPos = 0;
								
							if ((oneChar < ' ') || (oneChar > '~') || nowEsc)
								{			/* Rg[R[h */
									doControlCode(oneChar, &getCommand, &nowEsc, &nowCSI);
								}
							else
								{			/* LN^R[h */
									if (gDMem.CommandLength < kCommandBufferSize)
										{
											gDMem.CurrentCommandLine[gDMem.CommandLength++] = oneChar;
											sendChar(oneChar);
										}
								}
						}
					
					SCI3.SCR3.BIT.RIE = 1;		/* M֎~ԏI */
					
					if (getCommand)
						return;
				}
		}
}


/* R}h */
static void spritCommand(void)
{
	/*var*/
	SInt16							counter;
	SInt16							charCounter;
	Boolean							nowBreaker;
	SInt8								oneChar;

	/*functon*/
	/* S[hNA */
	for (counter = 0; counter < kMaxWordNum; counter++)
		gDMem.CurrentWords[counter][0] = 0x00;
	gDMem.NumWords = 0;
	charCounter = 0;
	
	/* 擪̓u[J[H */
	oneChar = gDMem.CurrentCommandLine[0];
	nowBreaker = (Boolean)((oneChar == ' ') || (oneChar == kTabCode));
	
	/* R}h[h̐擪TăRs[ */
	for (counter = 0; counter < gDMem.CommandLength; counter++)
		{
			oneChar = gDMem.CurrentCommandLine[counter];
		
			if ((oneChar == ' ') || (oneChar == kTabCode))
				{
					if (! nowBreaker)
						{
							gDMem.CurrentWords[gDMem.NumWords][charCounter] = 0x00;
							gDMem.NumWords++;
							charCounter = 0;
							nowBreaker = true;
						}
				}
			else
				{
					nowBreaker = false;
					gDMem.CurrentWords[gDMem.NumWords][charCounter++] = oneChar;
				}
		}
	
	/* Ō̃[h̃Vs */
	if (! nowBreaker)
		{
			gDMem.CurrentWords[gDMem.NumWords][charCounter] = 0x00;
			gDMem.NumWords++;
		}
}


/* R}hs */
static void	execCommand(void)
	{
		/*var*/
		SInt16							counter;
		SInt8*							theCommand;
		Boolean							found;
	
		/*functon*/
		/* R}ĥݑ啶 */
		theCommand = &(gDMem.CurrentWords[0][0]);
		UpperString(theCommand);
		
		/* R}hs */
		found = false;
		for (counter = 0; counter < kNumCommands; counter++)
			if (CompString(theCommand, CommandTable[counter].command))
				{
					CommandTable[counter].JumpAddress();
					found = true;
					break;
				}
		
		/* G[nhO */
		if (! found)
			{
				sendString(theCommand, false);
				sendString(kErrorNotSupportCommand, true);
			}
	}


/* j^Cxg҂[v */
static void	monitorMain(void)
	{
		/*var*/
		UInt16							ButtonCommand;
	
		/*functon*/
		ButtonCommand = 0;
		for ( ; ;)
			{
				/* R}hvvg\ */
				if (ButtonCommand == 0)
					sendString(kPrompt, false);
				
				/* R}h͑҂ */
				commandWait(&ButtonCommand);
				
				/* R}h́^s */
				switch(ButtonCommand)
					{
						case 0x01:
							// {^sRgɏ
							kBoardLED1 = 1;
							break;
						
						case 0x00:
						default:
							/* ^[~iR}h́^s */
							sendString(NULL, true);
							spritCommand();
							execCommand();
							break;
					}
			}
	}


/* C[` */
extern int main(void)
	{
		initialize();
		sendWelcome();
		monitorMain();
		return(0);
	}
	
